@@ -16,6 +16,7 @@ __PACKAGE__->setup_plugins([qw/
 /]);
 
 __PACKAGE__->config(
+  default_view => 'HTML',
   disable_component_resolution_regex_fallback => 1,
   using_frontend_proxy => 1,
   'Plugin::Session' => { storage_secret_key => 'abc123' },
@@ -31,12 +32,6 @@ __PACKAGE__->config(
 
 __PACKAGE__->setup();
   
-has users => (
-  is => 'ro',
-  lazy => 1,
-  default => sub($c) { $c->model('Schema::Person') },
-);
-
 has user => (
   is => 'rw',
   lazy => 1,
@@ -47,17 +42,13 @@ has user => (
 
 # This should probably return an empty user rather than undef
 sub get_user_from_session($self) {
-  my $id = $self->session->{user_id} // return $self->users->unauthenticated_user;
-  my $person = $self->users->find_by_id($id) // $self->remove_user_from_session && die "Bad ID '$id' in session";
+  my $id = $self->model('Session')->user_id // return $self->model('Schema::Person')->unauthenticated_user;
+  my $person = $self->model('Schema::Person')->find_by_id($id) // $self->logout && die "Bad ID '$id' in session";
   return $person;
 }
 
 sub persist_user_to_session ($self, $user) {
-  $self->session->{user_id} = $user->id;
-}
-
-sub remove_user_from_session($self) {
-  delete $self->session->{user_id};
+  $self->model('Session')->user_id($user->id);
 }
 
 sub authenticate($self, @args) {
@@ -67,15 +58,12 @@ sub authenticate($self, @args) {
 }
 
 sub logout($self) {
-  $self->remove_user_from_session;
+  $self->model('Session')->logout;
   $self->clear_user;
 }
 
-sub build_view($self, @args) {
-  my ($view_name) = @{$self->action->attributes->{View}};
-  my $view = $self->view($view_name, @args);
-  $self->stash(current_view_instance=>$view);
-  return $view;
+sub next_action($self, @args) {
+  $self->stash->{__received_args} = [@args];
 }
 
 __PACKAGE__->meta->make_immutable();
