@@ -27,13 +27,20 @@ sub _instantiate_builder {
   
   my %args = (
     model => $object,
-    name => $model_name,
+    name => ($model_name // _model_name_from_object_or_class($object)->param_key),
     options => $options
   );
 
   $args{namespace} = $options->{namespace} if exists $options->{namespace};
   $args{id} = $options->{id} if exists $options->{id};
   $args{index} = $options->{index} if exists $options->{index};
+  $args{parent_builder} = $options->{parent_builder} if exists $options->{parent_builder};
+  $args{theme} = $options->{theme} if exists $options->{theme};
+
+  if( exists($options->{parent_builder}) && exists($options->{parent_builder}{theme}) ) {
+    $args{theme} = +{ %{$args{theme}||+{}}, %{$options->{parent_builder}{theme}} };
+  }
+
   return Module::Runtime::use_module($builder)->new(%args);
 }
 
@@ -97,7 +104,7 @@ sub form_for {
   my $content_block_coderef = pop; # required; at the end
   my $options = @_ ? shift : +{};
   my $model_name = exists $options->{as} ? $options->{as} : _model_name_from_object_or_class($model)->param_key;
-  
+   
   _apply_form_options($model, $options);
 
   my $html_options = $options->{html};
@@ -130,18 +137,27 @@ sub form_for {
   }
 
   return Valiant::HTML::FormTags::form_tag $html_options, sub { 
-    my $captured = Valiant::HTML::FormTags::capture($content_block_coderef, $builder);
+    my $captured = Valiant::HTML::FormTags::capture($content_block_coderef, $builder, $model);
     $captured = $captured->concat(Valiant::HTML::FormTags::hidden_tag('csrf_token', {value=>$csrf_token})) if $csrf_token; 
     return $captured;
   };
 }
 
 #fields_for($name, $model, $options, sub {
-
 sub fields_for {
-  my ($name, $model, $options, $block) = @_;
+  my ($name, $model);
+  my $proto = shift;
+  if( Scalar::Util::blessed $proto ) {
+    $model = $proto;
+  } else {
+    $name = $proto;
+    $model = shift;
+  }
+  my $block = pop @_;
+  my $options = @_ ? shift(@_) : +{};
+  
   my $builder = _instantiate_builder($name, $model, $options);
-  return Valiant::HTML::FormTags::capture($block, $builder); 
+  return Valiant::HTML::FormTags::capture($block, $builder, $model); 
 }
 
 1;
@@ -176,7 +192,7 @@ Wrap a formbuilder object around it and generate HTML form field controls:
     my $person = Local::Person->new(first_name=>'J', last_name=>'Napiorkowski');
     $person->validate;
 
-    form_for($person, sub($fb) {
+    form_for($person, sub($fb, $person) {
       return  $fb->label('first_name'),
               $fb->input('first_name'),
               $fb->errors_for('first_name', +{ class=>'invalid-feedback' }),
@@ -284,7 +300,7 @@ The following functions can be exported by this library
 
 =head2 form_for
 
-    form_for($person, sub($fb) {
+    form_for($person, sub($fb, $person) {
       $fb->input('name');
       $fb->label('name');
     });
@@ -360,7 +376,7 @@ encoding issues.
 
 =head2 fields_for
 
-    fields_for($sub_model_name, $model, $options, sub($fb) {
+    fields_for($sub_model_name, $model, $options, sub($fb, $model) {
       $fb->input($field);
     });
 
@@ -371,6 +387,9 @@ Unless you are doing very customized form generation you'll probably use this as
 such as L<Valiant::HTML::FormBuilder>.  However there was no reason for me to not expose the method
 publically for users who need it.
 
+The second argument for the callback subroutine reference is the current model.  If the passed model is
+a collection this will always be the current one.
+
 =head1 SEE ALSO
  
 L<Valiant>, L<Valiant::HTML::FormBuilder>
